class Ratio {
    friend Ratio operator+(Ratio, Ratio);
    friend Ratio operator-(Ratio, Ratio);
    friend Ratio operator*(Ratio, Ratio);
    friend Ratio operator/(Ratio, Ratio);
    friend std::istream& operator>>(std::istream&, Ratio&);
    friend std::ostream& operator<<(std::ostream&, const Ratio&);
public:
    Ratio(long long, long long);       // constructor
    Ratio(const Ratio&); //copy constructor
private:
    void _reduce();        // enforces the class invariants
    long long _num;              // numerator
    long long _den;              // denominator
};

Ratio operator+(Ratio x, Ratio y) {
    long long num = x._num*y._den + x._den*y._num;
    long long den = x._den*y._den;
    Ratio temp(num, den);
    temp._reduce();
    return temp;
}

Ratio operator-(Ratio x, Ratio y) {
    long long num = x._num*y._den - x._den*y._num;
    long long den = x._den*y._den;
    Ratio temp(num, den);
    temp._reduce();
    return temp;
}

Ratio operator*(Ratio x, Ratio y) {
    long long num = x._num*y._num;
    long long den = x._den*y._den;
    Ratio temp(num, den);
    temp._reduce();
    return temp;
}

Ratio operator/(Ratio x, Ratio y) {
    long long num = x._num*y._den;
    long long den = x._den*y._num;
    Ratio temp(num, den);
    temp._reduce();
    return temp;
}

std::ostream& operator<<(std::ostream& ostr, const Ratio& x) {
    if (x._den == 1) ostr << x._num;
    else ostr << x._num << '/' << x._den;
    return ostr;
}

std::istream& operator>>(std::istream& istr, Ratio& x) {
    char ch;         // used to eat the slash character '/'
    istr >> x._num;
    if (istr.peek() == '/') istr >> ch >> x._den;
    else x._den = 1;
    x._reduce();
    return istr;
}

//Ratio::Ratio(long long num=0, long long den=0) {
//    _num = num;
//    _den = den;
//    _reduce();
//}

Ratio::Ratio(long long num=0, long long den=1) : _num(num), _den(den) {
    _reduce();
}

Ratio::Ratio(const Ratio& r) : _num(r._num), _den(r._den) {}


long long gcd(long long m, long long n) {
    assert(m > 0 && n > 0);
    while (m > 0) {
        if (m < n) std::swap(m,n);
        m -= n;
    }
    return n;
}

void Ratio::_reduce() {
    if (_num == 0 || _den == 0) {
        _num = 0;
        _den = 1;
        return;
    }
    if (_den < 0)       // enforces constralong long: _den > 0
    {   _num *= -1;
        _den *= -1;
    }
    long long abs_num = (_num < 0 ? -_num : _num);             // = |_num|
    long long g = gcd(abs_num, _den);
    _num /= g;  // enforces constraint that ratio be in lowest terms
    _den /= g;
}
